@@ -14,14 +14,14 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 # Class:
-# 
+#
 #   EBox::DNS::Model::DomainTable
 #
-#   This class inherits from <EBox::Model::DataTable> and represents the
-#   object table which basically contains domains names and a reference
-#   to a member <EBox::DNS::Model::HostnameTable>
+#   This class inherits from <EBox::Model::DataTable> and represents
+#   the object table which basically contains domains names, the IP
+#   address for the domain and a reference to a member
+#   <EBox::DNS::Model::HostnameTable>
 #
-#   
 package EBox::DNS::Model::DomainTable;
 
 use EBox::Global;
@@ -30,8 +30,9 @@ use EBox::Validate qw(:all);
 use EBox::Exceptions::External;
 use EBox::Exceptions::DataExists;
 
-use EBox::Types::Text;
+use EBox::Types::DomainName;
 use EBox::Types::HasMany;
+use EBox::Types::HostIP;
 use EBox::Sudo;
 
 use EBox::Model::ModelManager;
@@ -43,7 +44,7 @@ use warnings;
 
 use base 'EBox::Model::DataTable';
 
-sub new 
+sub new
 {
     my $class = shift;
     my %parms = @_;
@@ -56,10 +57,10 @@ sub new
 
 sub _table
 {
-    my @tableHead = 
-        ( 
+    my @tableHead =
+        (
 
-            new EBox::Types::Text
+            new EBox::Types::DomainName
                             (
                                 'fieldName' => 'domain',
                                 'printableName' => __('Domain'),
@@ -73,13 +74,29 @@ sub _table
                                 'printableName' => __('Hostnames'),
                                 'foreignModel' => 'HostnameTable',
                                 'view' => '/ebox/DNS/View/HostnameTable',
-                                'backView' => '/ebox/DNS/View/HostnameTable',
+                                'backView' => '/ebox/DNS/View/DomainTable',
                                 'size' => '1',
-                             )
+                             ),
+            new EBox::Types::HasMany
+                            (
+                                'fieldName' => 'mailExchangers',
+                                'printableName' => __('Mail Exchangers'),
+                                'foreignModel' => 'MailExchanger',
+                                'view' => '/ebox/DNS/View/MailExchanger',
+                                'backView' => '/ebox/DNS/View/DomainTable',
+                             ),
+            new EBox::Types::HostIP
+                            (
+                                'fieldName' => 'ipaddr',
+                                'printableName' => __('IP Address'),
+                                'size' => '20',
+                                'optional' => 1,
+                                'editable' => 1
+                            ),
           );
 
-    my $dataTable = 
-        { 
+    my $dataTable =
+        {
             'tableName' => 'DomainTable',
             'printableTableName' => __('Domains'),
             'automaticRemove' => 1,
@@ -89,22 +106,12 @@ sub _table
             'class' => 'dataTable',
             'help' => __('Domains'),
             'printableRowName' => __('domain'),
+            'sortedBy' => 'domain',
         };
 
     return $dataTable;
 }
 
-# Method: validateRow
-#
-#      Override <EBox::Model::DataTable::validateRow> method
-#
-sub validateRow()
-{
-    my ($self, $action, %param) = @_;
-
-		checkDomainName($param{'domain'}, __("Domain name"));
-}
-
 # Method: addDomain
 #
 #   Add a domain to the domainss table. Note this method must exist
@@ -114,26 +121,26 @@ sub validateRow()
 # Parameters:
 #
 #   (NAMED)
-#   name       - domain's name
+#   domain_name - String domain's name
 #   hostnames  - array ref containing the following hash ref in each value:
 #
 #                name        - host's name
-#                ip          - hostr's ipaddr 
+#                ip          - hostr's ipaddr
 #                aliases     - array ref containing alias names;
 #
 #   Example:
 #
-#       name      => 'foo.com',
-#       hostnames => [ 
-#                     { 'name'         => 'bar',
-#                       'ip'           => '192.168.1.2',
-#                       'aliases'      => [
-#                                        { 'bar',
-#                                          'b4r'
-#                                        }
-#                                       ]
-#                     }
-#                    ]
+#       domain_name => 'foo.com',
+#       hostnames   => [
+#                       { 'name'         => 'bar',
+#                         'ip'           => '192.168.1.2',
+#                         'aliases'      => [
+#                                             { 'bar',
+#                                               'b4r'
+#                                             }
+#                                           ]
+#                       }
+#                      ]
 sub addDomain
 {
     my ($self, $params) = @_;
