@@ -14,14 +14,13 @@
 # Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
 # Class:
-# 
+#
 #   EBox::DNS::Model::HostnameTable
 #
-#   This class inherits from <EBox::Model::DataTable> and represents the
-#   object table which basically contains domains names and a reference
-#   to a member <EBox::DNS::Model::HostnameTable>
+#   This class inherits from <EBox::Model::DataTable> and represents
+#   the host names (A resource records) in a domain and a set of alias
+#   described in <EBox::Network::Model::AliasTable>
 #
-#   
 package EBox::DNS::Model::HostnameTable;
 
 use EBox::Global;
@@ -30,8 +29,9 @@ use EBox::Validate qw(:all);
 use EBox::Exceptions::External;
 use EBox::Exceptions::DataExists;
 
-use EBox::Types::Text;
+use EBox::Types::DomainName;
 use EBox::Types::HasMany;
+use EBox::Types::HostIP;
 use EBox::Sudo;
 
 use EBox::Model::ModelManager;
@@ -43,7 +43,9 @@ use warnings;
 
 use base 'EBox::Model::DataTable';
 
-sub new 
+# Group: Public methods
+
+sub new
 {
     my $class = shift;
     my %parms = @_;
@@ -54,66 +56,6 @@ sub new
     return $self;
 }
 
-sub _table
-{
-    my @tableHead = 
-        ( 
-
-            new EBox::Types::Text
-                            (
-                                'fieldName' => 'hostname',
-                                'printableName' => __('Hostname'),
-                                'size' => '20',
-                                'unique' => 1,
-                                'editable' => 1
-                             ),
-            new EBox::Types::Text
-                            (
-                                'fieldName' => 'ipaddr',
-                                'printableName' => __('IP Address'),
-                                'size' => '20',
-                                'unique' => 1,
-                                'editable' => 1
-                             ),
-            new EBox::Types::HasMany
-                            (
-                                'fieldName' => 'alias',
-                                'printableName' => __('Alias'),
-                                'foreignModel' => 'AliasTable',
-                                'view' => '/ebox/DNS/View/AliasTable',
-                                'backView' => '/ebox/DNS/View/AliasTable',
-                                'size' => '1',
-                             )
-          );
-
-    my $dataTable = 
-        { 
-            'tableName' => 'HostnameTable',
-            'printableTableName' => __('Hostnames'),
-            'automaticRemove' => 1,
-            'defaultController' => '/ebox/Dns/Controller/HostnameTable',
-            'defaultActions' => ['add', 'del', 'editField',  'changeView' ],
-            'tableDescription' => \@tableHead,
-            'class' => 'dataTable',
-            'help' => __('Hostnames'),
-            'printableRowName' => __('hostname'),
-        };
-
-    return $dataTable;
-}
-
-# Method: validateRow
-#
-#      Override <EBox::Model::DataTable::validateRow> method
-#
-sub validateRow()
-{
-    my ($self, $action, %param) = @_;
-
-		checkDomainName($param{'hostname'}, __("Domain name"));
-		checkIP($param{'ipaddr'}, __("IP address"));
-}
-
 # Method: addHostname
 #
 #   Add a hostname to the hostnames table. Note this method must exist
@@ -123,13 +65,13 @@ sub validateRow()
 # Parameters:
 #
 #   (NAMED)
-#   name       - host name
-#   ip         - host ipaddr
+#   hostname   - String host name
+#   ip         - String host ipaddr
 #   aliases    - array ref containing the alias names
 #
 #   Example:
 #
-#      'name'         => 'bar',
+#      'hostname'     => 'bar',
 #      'ip'           => '192.168.1.2',
 #      'aliases'      => [
 #                         { 'bar',
@@ -148,7 +90,7 @@ sub addHostname
    my $id = $self->addRow('hostname' => $name, 'ipaddr' => $ip);
 
    unless (defined($id)) {
-       throw EBox::Exceptions::Internal("Couldn't add hostname: $name");
+       throw EBox::Exceptions::Internal("Couldn't add host name: $name");
    }
 
    my $aliases = delete $params{'aliases'};
@@ -162,4 +104,117 @@ sub addHostname
        $aliasModel->addRow('alias' => $alias);
    }
 }
+
+# Method: validateTypedRow
+#
+# Overrides:
+#
+#    <EBox::Model::DataTable::validateTypedRow>
+#
+# Exceptions:
+#
+#    <EBox::Exceptions::External> - thrown if there is an alias with
+#    the same name for other hostname within the same domain
+#
+sub validateTypedRow
+{
+    my ($self, $action, $changedFields, $allFields) = @_;
+
+    return unless ( exists $changedFields->{hostname} );
+
+    # Check there is no CNAME RR in the domain with the same name
+    my $newHostName = $changedFields->{hostname};
+    my $domainModel = $changedFields->{hostname}->row()->model();
+
+    for my $id (@{$domainModel->ids()}) {
+        my $row = $domainModel->row($id);
+        for my $id (@{$row->subModel('alias')->ids()}) {
+           my $subRow = $row->subModel('alias')->row($id);
+           if ($subRow->elementByName('alias')->isEqualTo($newHostName)) {
+                throw EBox::Exceptions::External(
+                    __x('There is an alias with the same name "{name}" '
+                        . 'for "{hostname}" in the same domain',
+                         name     => $subRow->valueByName('alias'),
+                         hostname => $row->valueByName('hostname')));
+            }
+        }
+    }
+}
+
+# Group: Protected methods
+
+# Method: _table
+#
+# Overrides:
+#
+#    <EBox::Model::DataTable::_table>
+#
+sub _table
+{
+    my @tableHead =
+        (
+            new EBox::Types::DomainName
+                            (
+                                'fieldName' => 'hostname',
+                                'printableName' => __('Host name'),
+                                'size' => '20',
+                                'unique' => 1,
+                                'editable' => 1
+                             ),
+            new EBox::Types::HostIP
+                            (
+                                'fieldName' => 'ipaddr',
+                                'printableName' => __('IP Address'),
+                                'size' => '20',
+                                'unique' => 1,
+                                'editable' => 1
+                             ),
+            new EBox::Types::HasMany
+                            (
+                                'fieldName' => 'alias',
+                                'printableName' => __('Alias'),
+                                'foreignModel' => 'AliasTable',
+                                'view' => '/ebox/DNS/View/AliasTable',
+                                'backView' => '/ebox/DNS/View/AliasTable',
+                                'size' => '1',
+                             )
+          );
+
+    my $dataTable =
+        {
+            'tableName' => 'HostnameTable',
+            'printableTableName' => __('Host names'),
+            'automaticRemove' => 1,
+            'defaultController' => '/ebox/Dns/Controller/HostnameTable',
+            'defaultActions' => ['add', 'del', 'editField',  'changeView' ],
+            'tableDescription' => \@tableHead,
+            'class' => 'dataTable',
+            'help' => __('Hostnames'),
+            'printableRowName' => __('host name'),
+            'sortedBy' => 'hostname',
+        };
+
+    return $dataTable;
+}
+
+# Method: deletedRowNotify
+#
+# 	Overrides <EBox::Model::DataTable::deletedRowNotify> to remove
+# 	mail exchangers referencing the deleted host name.
+#
+sub deletedRowNotify
+{
+    my ($self, $row) = @_;
+
+    my $mailExModel = $row->parentRow()->subModel('mailExchangers');
+    for my $id(@{$mailExModel->ids()}) {
+        my $mailRow = $mailExModel->row($id);
+        my $hostname = $mailRow->elementByName('hostName');
+        next unless ($hostname->selectedType() eq 'ownerDomain');
+        if ($hostname->value() eq $row->id()) {
+            $mailExModel->removeRow($mailRow->id());
+        }
+    }
+}
+
 1;
